@@ -32,13 +32,12 @@ This WSGI component:
 """
 
 import logging
-import webob
 
+from oslo.serialization import jsonutils
 import requests
 import six
 from six.moves import urllib
-
-from keystoneclient.openstack.common import jsonutils
+import webob
 
 
 PROTOCOL_NAME = 'S3 Token Authentication'
@@ -103,6 +102,10 @@ class S3Token(object):
         self.app = app
         self.logger = logging.getLogger(conf.get('log_name', __name__))
         self.logger.debug('Starting the %s component', PROTOCOL_NAME)
+        self.logger.warning(
+            'This middleware module is deprecated as of v0.11.0 in favor of '
+            'keystonemiddleware.s3_token - please update your WSGI pipeline '
+            'to reference the new middleware package.')
         self.reseller_prefix = conf.get('reseller_prefix', 'AUTH_')
         # where to find the auth service (we use this to validate tokens)
 
